/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms;

import java.io.InputStream;
import java.util.List;

/**
 * Models a cms node
 */
public class CmsNode {

	String name = "";
	String path = "";
	String version = "";
	String uuid = "";
	String type = "";
	List versions = null;
	List childs = null;
	List properties = null;
	InputStream content = null;
	
	/**
	 * Constructor of the class useful to create a node setting its name, path, 
	 * current version name, uuid and type
	 * @param namein the name of the node
	 * @param pathin the path of the node
	 * @param versionin the current version name
	 * @param uuidin the uuid of the node
	 * @param typein the type of the node
	 */
	public CmsNode(String namein, String pathin, String versionin, String uuidin, String typein) {
		name = namein;
		path = pathin;
		version = versionin;
		uuid = uuidin;
		type = typein;
	}
	
	/**
	 * Constructor of the class useful to create a node setting its name, path, 
	 * current version name, uuid, type and content
	 * @param namein the name of the node
	 * @param pathin the path of the node
	 * @param versionin the current version name
	 * @param uuidin the uuid of the node
	 * @param typein the type of the node
	 * @param contin the InputStream content of the node
	 */
	public CmsNode(String namein, String pathin, String versionin, String uuidin, 
			       String typein, InputStream contin) {
		name = namein;
		path = pathin;
		version = versionin;
		uuid = uuidin;
		type = typein;
		content = contin;
	}
	
	/**
	 * Constructor of the class useful to create a node setting its name, path, 
	 * current version name, uuid, type, list of childs,
	 * list of properties, list of versions and content
	 * @param namein the name of the node
	 * @param pathin the path of the node
	 * @param versionin the current version name
	 * @param uuidin the uuid of the node
	 * @param typein the type of the node
	 * @param chidsin the list of the node childs 
	 * Each child must be of type it.eng.spago.cms.CmsNode
	 * @param propsin the list of the node properties 
	 * Each property must be of type it.eng.spago.cms.CmsProperty
	 * @param verin the list of the node versions 
	 * Each version must be of type it.eng.spago.cms.CmsVersion
	 * @param contin the InputStream content of the node
	 */
	public CmsNode(String namein, String pathin, String versionin, String uuidin, String typein,
			       List childsin, List propsin, List versin, InputStream contin) {
		name = namein;
		path = pathin;
		version = versionin;
		uuid = uuidin;
		type = typein;
		childs = childsin;
		properties = propsin;
		versions = versin;
		content = contin;
	}
	
	
	/**
	 * Constructor of the class useful to create a node setting its name, path, 
	 * current version name, uuid, type, list of childs,
	 * list of properties and list of versions
	 * @param namein the name of the node
	 * @param pathin the path of the node
	 * @param versionin the current version name
	 * @param uuidin the uuid of the node
	 * @param typein the type of the node
	 * @param chidsin the list of the node childs 
	 * Each child must be of type it.eng.spago.cms.CmsNode
	 * @param propsin the list of the node properties 
	 * Each property must be of type it.eng.spago.cms.CmsProperty
	 * @param verin the list of the node versions 
	 * Each version must be of type it.eng.spago.cms.CmsVersion
	 */
	public CmsNode(String namein, String pathin, String versionin, String uuidin, String typein,
		           List childsin, List propsin, List versin) {
	name = namein;
	path = pathin;
	version = versionin;
	uuid = uuidin;
	type = typein;
	childs = childsin;
	properties = propsin;
	versions = versin;
}
	
	/**
	 * Gets the list of the node childs
	 * @return The list of the child nodes 
	 * Each child is an instance of type it.eng.spago.cms.CmsNode
	 */
	public List getChilds() {
		return childs;
	}
	/**
	 * Sets the childs list
	 * @param childs The childs list. Each child must be an instance of
	 * type it.eng.spago.cms.CmsChild
	 */
	public void setChilds(List childs) {
		this.childs = childs;
	}
	/**
	 * Gets the content of the node
	 * @return The InputStream of the node content
	 */
	public InputStream getContent() {
		return content;
	}
	/**
	 * Sets the content of the node
	 * @param content The content of the node
	 */
	public void setContent(InputStream content) {
		this.content = content;
	}
	/**
	 * Gets the name of the node
	 * @return the name of the node
	 */
	public String getName() {
		return name;
	}
	/**
	 * Sets the name of the node
	 * @param name The name of the node
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * Gets the path of the node
	 * @return the path of the node
	 */
	public String getPath() {
		return path;
	}
	/**
	 * Sets the path of the node
	 * @param path The path of the node
	 */
	public void setPath(String path) {
		this.path = path;
	}
	/**
	 * Gets the list of the node properties
	 * @return The list of the child properties 
	 * Each property is an instance of type it.eng.spago.cms.CmsProperty
	 */
	public List getProperties() {
		return properties;
	}
	/**
	 * Sets the list of node properties
	 * @param properties The list of node properties. Each property mmust be an
	 * instance of type it.eng.spago.cms.CmsProperty
	 */
	public void setProperties(List properties) {
		this.properties = properties;
	}
	/**
	 * Gets the type of the node
	 * @return the type of the node
	 */
	public String getType() {
		return type;
	}
	/**
	 * Sets the type
	 * @param type The type of the node
	 */
	public void setType(String type) {
		this.type = type;
	}
	/**
	 * Gets the uuid of the node
	 * @return The uuid of the node
	 */
	public String getUuid() {
		return uuid;
	}
	/**
	 * Sets the uuid
	 * @param uuid The uuid of the node
	 */
	public void setUuid(String uuid) {
		this.uuid = uuid;
	}
	/**
	 * Gets the current version name
	 * @return The current version name
	 */
	public String getVersion() {
		return version;
	}
	/**
	 * Sets tbe name of the current version
	 * @param version The name of the current version
	 */
	public void setVersion(String version) {
		this.version = version;
	}
	/**
	 * Gets the list of the node versions
	 * @return The list of the child versions 
	 * Each version is an instance of type it.eng.spago.cms.CmsVersion
	 */
	public List getVersions() {
		return versions;
	}
	/**
	 * Sets the list of the node versions
	 * @param versions The list of the node versions. Each version must be an 
	 * instance of the type it.eng.spago.cms.CmsVersion
	 */
	public void setVersions(List versions) {
		this.versions = versions;
	}
	
	
	
}
