package it.eng.spago.cms.exec;

import javax.jcr.Session;

import it.eng.spago.base.Constants;
import it.eng.spago.cms.init.CMSManager;
import it.eng.spago.tracing.TracerSingleton;


/**
 * Models a connection to the jsr 170 repository
 */
public class CMSConnection {
    
	/**
	 * Spago CMS namespace prefix for system properties
	 */
	private String prefixPropertySystem;
	/**
	 * Spago CMS namespace prefix for user properties
	 */
	private String prefixPropertyUser;
	/**
	 * JCR Session
	 */
	private Session session;
	/**
	 * flag for close / open state of the connection
	 */
	private boolean close = false;
	/**
	 * Name of the repository connection pool to which the connection belongs
	 */
	private String nameRepositoryPool = null;
	

	/**
	 * Constructor of the connection
	 * @param sess, jcr session
	 * @param nameRep, name of the repository connect
	 * @param preSys, prefix for system properties
	 * @param preUsr, prefix for user properties
	 */
	public CMSConnection(Session sess, String nameRep, String preSys, String preUsr) {
		prefixPropertySystem = preSys;
		prefixPropertyUser = preUsr;
		nameRepositoryPool = nameRep;
		session = sess;
	}
		
	
	/**
	 * Closes the connection
	 */
	public void close() {
	    try {
	    	CMSManager.getInstance(nameRepositoryPool).closeConnection(this);
		    close = true;
	    }
	    catch (Exception e) {
	    	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"CMSConnection::close: cannot close the connection");			
	    }
	}
	
	/**
	 * Checks if the connection is close
	 * @return boolean, true if connection is close false otherwise
	 */
	public boolean isClose() {
		return close;
	}
	
	/**
	 * Gets a jcr 170 session 
	 * @return a jcr 170 Session
	 */
	public Session getSession() {
		return session;
	}
	
	/**
	 * Gets the name of the repository connection pool to which the connection belongs
	 * @return Name of the repository connection pool to which the connection belongs
	 */
	public String getNameRepositoryPool() {
		return nameRepositoryPool;
	}
	
	/**
	 * Gets the prefix used for system properties
	 * @return Spago CMS system properties prefix
	 */
	public String getPrefixPropertySystem() {
		return prefixPropertySystem;
	}
	
	/**
	 * Gets the prefix used for user properties
	 * @return Spago CMS user properties prefix
	 */
	public String getPrefixPropertyUser() {
		return prefixPropertyUser;
	}
	
	/**
	 * Set the close / open state of the connection
	 * @param close state to set
	 */
	public void setClose(boolean close) {
		this.close = close;
	}
}
