package it.eng.spago.cms.exec;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.Utils;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.tracing.TracerSingleton;

import java.io.IOException;
import java.io.OutputStream;

import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;


/**
 * Implements method to perform an export operation
 * into the repository
 */
public class ExportObjectImpl {

	protected void execute(SourceBean opConf, CMSConnection conn) throws OperationExecutionException {
		Session session = conn.getSession();
		String pathStr = null;
		OutputStream out = null;
		boolean noRecurse = false;
		String method = "ExportObjectImpl::execute(SourceBean,CmsConnection):";
		try {
			// get the path of the node to be exported
		    pathStr = OperationDescriptorUtils.getStringAttribute(opConf, OperationsConstants.PATH);
		    Path path = Path.create(pathStr);
		    // get the output stream for the nodes serialization
		    out = (OutputStream) OperationDescriptorUtils.getObjectAttribute(opConf, OperationsConstants.STREAMATTR);
		    // get the noRecurse flag
		    noRecurse = OperationDescriptorUtils.getBooleanAttribute(opConf, OperationsConstants.NORECURSE).booleanValue();
		    Utils.debug(method + " Starting export operation of node at path " +
		    		"" + path.getAbsPathStr() + " with noRecurse = [" 
		    		+ noRecurse + "] ...");
		    session.exportSystemView(path.getAbsPathStr(), out, false, noRecurse);
		    Utils.debug(method + " Export operation of node at path " + path.getAbsPathStr()
		    		+ " successfully performed");
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    			"ExportObjectImpl::execute: object not found", e);	
			throw new OperationExecutionException("ExportObjectImpl::execute: object not found");
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    			"ExportObjectImpl::execute: repository access error", e);
			throw new OperationExecutionException("ExportObjectImpl::execute: repository acess error");
		} catch (IOException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
					"ExportObjectImpl::execute: I/O error during exporting operation", e);
			throw new OperationExecutionException("ExportObjectImpl::execute: I/O error during exporting operation");
		} catch (PathNotValidException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
					"ExportObjectImpl::execute: Path is not valid", e);
			throw new OperationExecutionException("ImportObjectImpl::execute: Path is not valid");
		}
	}
	
}
