package it.eng.spago.cms.exec;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.Utils;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.tracing.TracerSingleton;

import java.io.IOException;
import java.io.InputStream;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;


/**
 * Implements method to perform an import operation into the repository.
 */
public class ImportObjectImpl {

	protected void execute(SourceBean opConf, CMSConnection conn) throws OperationExecutionException {
		Session session = conn.getSession();
		String pathStr = null;
		InputStream in = null;
		int uuidBehavior;
		String method = "ImportObjectImpl::execute(SourceBean,CmsConnection):";
		try {
			// get the path of the node were to place incoming nodes
		    pathStr = OperationDescriptorUtils.getStringAttribute(opConf, OperationsConstants.PATH);
		    // get the input stream for the nodes deserialization
		    in = (InputStream) OperationDescriptorUtils.getObjectAttribute(opConf, OperationsConstants.STREAMATTR);
		    // get the uuidBehavior flag
		    uuidBehavior = new Integer(OperationDescriptorUtils.getStringAttribute(opConf, 
		    		OperationsConstants.UUIDBEHAVIOR)).intValue();
		    String uuidBehaviourMsg = null;
		    switch (uuidBehavior) {
		    	case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING :
		    		uuidBehaviourMsg = "IMPORT_UUID_COLLISION_REMOVE_EXISTING";
		    		break;
		    	case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING :
		    		uuidBehaviourMsg = "IMPORT_UUID_COLLISION_REPLACE_EXISTING";
		    		break;
		    	case ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW :
		    		uuidBehaviourMsg = "IMPORT_UUID_COLLISION_THROW";
		    		break;
		    	case ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW :
		    		uuidBehaviourMsg = "IMPORT_UUID_CREATE_NEW";
		    		break;
		    	default :
		    		uuidBehaviourMsg = "UNKNOWN";
		    }
		    Path path = Path.create(pathStr);
		    Utils.debug(method + " Starting import operation at path " + path.getAbsPathStr() + " with uuidBehavior = [" 
		    		+ uuidBehaviourMsg + "] ...");
		    session.importXML(path.getAbsPathStr(), in, uuidBehavior);
		    session.save();
		    Utils.debug(method + " Import operation at path " + path.getAbsPathStr() + " successfully performed");
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    			"ImportObjectImpl::execute: object not found", e);	
			throw new OperationExecutionException("ImportObjectImpl::execute: object not found");
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    			"ImportObjectImpl::execute: repository access error", e);
			throw new OperationExecutionException("ImportObjectImpl::execute: repository acess error");
		} catch (IOException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
					"ImportObjectImpl::execute: I/O error during importing operation", e);
			throw new OperationExecutionException("ImportObjectImpl::execute: I/O error during importing operation");
		} catch (PathNotValidException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
					"ImportObjectImpl::execute: Path is not valid", e);
			throw new OperationExecutionException("ImportObjectImpl::execute: Path is not valid");
		} 
	}
	
}
