package it.eng.spago.cms.exec;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.Utils;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.tracing.TracerSingleton;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

/**
* Implements method to perform a version operation into the repository.
* It means that the node relevant to the path attribute of the input SourceBean
* is versioned only if it is versionable; if the node isn't versionable, an exception is thrown.
*/
public class VersionObjectImpl {

	protected void execute(SourceBean opConf, CMSConnection conn) throws OperationExecutionException {
		Session session = conn.getSession();
		String pathStr = null;
		boolean noRecurse = false;
		String method = "VersionObjectImpl::execute(SourceBean,CmsConnection):";
	    Path path;
		try {
			// get the path of the node to be versioned
			pathStr = OperationDescriptorUtils.getStringAttribute(opConf, OperationsConstants.PATH);
			path = Path.create(pathStr);
			noRecurse = OperationDescriptorUtils.getBooleanAttribute(opConf, OperationsConstants.NORECURSE).booleanValue();
			// get the noRecurse flag
			Utils.debug(method + " Starting version operation of node at path " + path.getAbsPathStr() + 
					"with noRecurse flag = " + noRecurse);
			Node node = session.getRootNode().getNode(path.getRootRelativePathStr());
			if (!node.isNodeType("mix:versionable")) {
				throw new OperationExecutionException("VersionObjectImpl::execute: node is not versionable");
			} else {
				node.checkin();
				node.checkout();
	        }
			if (!noRecurse) {
				versionChildrenNodes(node);
			}
			Utils.debug(method + "Version operation of node at path " + path.getAbsPathStr() 
					+ " successfully performed");
		} catch (PathNotValidException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    			"VersionObjectImpl::execute: path not valid", e);	
			throw new OperationExecutionException("VersionObjectImpl::execute: path not valid");
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    			"VersionObjectImpl::execute: object not found", e);	
			throw new OperationExecutionException("VersionObjectImpl::execute: object not found");
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    			"VersionObjectImpl::execute: repository access error", e);
			throw new OperationExecutionException("VersionObjectImpl::execute: repository acess error");
		}
	    
	}
	
	private void versionChildrenNodes (Node node) throws RepositoryException {
		NodeIterator it = node.getNodes();
		while (it.hasNext()) {
			Node aChildrenNode = it.nextNode();
			if (aChildrenNode.isNodeType("mix:versionable")) {
				aChildrenNode.checkin();
				aChildrenNode.checkout();
			}
			versionChildrenNodes(aChildrenNode);
		}
	}
	
}
