package it.eng.spago.cms.pool;

import it.eng.spago.cms.IRepositoryProvider;
import it.eng.spago.cms.exec.CMSConnection;
import it.eng.spago.cms.util.Utils;

import javax.jcr.Session;

import org.apache.commons.pool.BasePoolableObjectFactory;

/**
 * Implements an interface of the jackarta commons pool library in order 
 * to create objects for the pool (see documentation of jackarta
 * commons pool) 
 */
public class ConnectionCMSPoolableObjectFactory extends  BasePoolableObjectFactory{
	
	IRepositoryProvider _repo = null;
	String nameRepositoryPool = null;
	String defaultUser = null;
	String defaultPassword = null;
	String defaultWorkspace = null;
	String prefSys = null;
	String prefUsr = null;
	
	/**
	 * Constructor of the class
	 * @param repo, jcr 170 repository
	 * @param name, name of the repository
	 * @param user, default user name used for open Session
	 * @param password, default password used for open session 
	 * @param workSpace, default workspace used for open session
	 * @param preSys, namespace prefix for system cms properties
	 * @param preUsr, namespace prefix for user cms properties
	 */
	protected ConnectionCMSPoolableObjectFactory(IRepositoryProvider repo, String name, 
			  String user, String password, String workSpace, String preSys, String preUsr ) {
		super();
		_repo = repo;
		nameRepositoryPool = name;
		defaultUser = user;
		defaultPassword = password;
		defaultWorkspace = workSpace;
		prefSys = preSys;
		prefUsr = preUsr;
	}
	
	/**
	 * Creates a new object to sotre into the pool (see documentation of jackarta
     * commons pool )
     * @return new Object for the pool
	 */
	public Object makeObject() {
		Object obj = null;
		try {
			Session sess = Utils.getSession(_repo, defaultUser, defaultPassword, defaultWorkspace);
			CMSConnection conn = new CMSConnection(sess, nameRepositoryPool, prefSys, prefUsr);
			obj = (Object)conn;
		}
		catch (Exception e) {
			obj = null;
		}
		return obj;
	}
}
