package it.eng.spago.cms.repositoryproviders;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.IRepositoryProvider;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;
import java.util.List;

import javax.jcr.Repository;
import javax.jcr.RepositoryException;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;

/**
 * Implements methods to get the JSR 170 Repository instance from an Exo ECM server.
 */
public class ExoRepositoryProvider implements IRepositoryProvider {

	/**
	 * Name of the module
	 */
	private static final String NAME_MODULE = "SPAGOCMS";
	
	/**
	 * JSR 170 Repository instance
	 */
	protected Repository _repo = null;

	
	/**
	 * Construct of the class, gets the JSR 170 repository object instance from an eXo ECM server.
	 * It needs a parameter which contains the name of the exo portal application context name, so, 
	 * into the Spago configuration files must be present an xml envelope structured like below:
	 * <pre>
	 * &lt;CONTENTCONFIGURATION&gt;
	 * 		&lt;CONTENTREPOSITORY&gt;
	 * 			&lt;PARAMETERS&gt;
	 * 					&lt;CONTEXTNAME name="name of the portal context name" /&gt;
	 * <pre>
	 */
	public ExoRepositoryProvider() {
		try{	
			String paramCont = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.CONTEXTNAME";
			ConfigSingleton config = ConfigSingleton.getInstance();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider::init:Spago configuration retrived");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider::init:try to get the "+paramCont+
								" parameter from the cms configuration ");
			SourceBean paramContSB = (SourceBean)config.getAttribute(paramCont);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider::init:cms sourcebean parameter context name retrived");
			String nameCont = (String)paramContSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider::init:use "+nameCont+" context name");
			RootContainer rootCont = RootContainer.getInstance();
			List comps = rootCont.getComponentInstances();
			Iterator iter = comps.iterator();
			while(iter.hasNext()) {
				Object comp = iter.next();
				if(comp instanceof PortalContainer ) {
					PortalContainer port = (PortalContainer)comp;
                    // Modifica necessaria per compilare con le librerie presenti su iBiblio
//					String name = port.getPortalContainerInfo().getContainerName();
//					System.out.println(name);
				}
			}
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider::init:Exo root container retrived");
			PortalContainer container = rootCont.getPortalContainer(nameCont); 
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider::init:Exo "+nameCont+" portal container retrived");
			PortalContainer.setInstance(container);
			RepositoryService repositoryService = 
				(RepositoryService)container.getComponentInstanceOfType(RepositoryService.class);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider:Exo repository service retrived");
			Repository repository = repositoryService.getRepository();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryProvider:Exo jcr repository retrived");
			_repo = repository;
		} catch (RepositoryException e) {
			TracerSingleton.log(NAME_MODULE,
					            TracerSingleton.CRITICAL,
					            "ExoRepositoryProvider::init: cannot get the repository", e);
		} catch (RepositoryConfigurationException e) {
			TracerSingleton.log(NAME_MODULE,
		            TracerSingleton.CRITICAL,
		            "ExoRepositoryProvider::init: cannot get the repository", e);
		} catch (Exception e) {
			TracerSingleton.log(NAME_MODULE,
		            TracerSingleton.CRITICAL,
		            "ExoRepositoryProvider::init: cannot get the repository", e);
		}finally {	}
	}
	
	/**
	 * Gets the JSR 170 repository object instance
	 * @param Repository the JSR 170 repository object instance 
	 */
	public Repository getRepository() {
		return _repo;
	}


}




