package it.eng.spago.cms.repositoryproviders;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.IRepositoryProvider;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import javax.jcr.Repository;

import org.apache.jackrabbit.rmi.client.ClientRepositoryFactory;


/**
 * Implements methods to get the JSR 170 Repository instance from an RMI server
 */
public class RmiRepositoryProvider implements IRepositoryProvider {

	/**
	 * Name of the module
	 */
	private static final String NAME_MODULE = "SPAGOCMS";
	
	/**
	 * JSR 170 Repository instance
	 */
	protected Repository _repo = null;

	
	/**
	 * Construct of the class, gets the JSR 170 repository object instance from an RMI server.
	 * Obviously the RMI level must have been configured before.
	 * It needs a parameter: 
	 * <ul>
	 * 		<li>rmi name: the rmi name of the jsr 170 repository</li>
	 * </ul> 
	 * into the Spago configuration files must be present an xml envelope structured like below:
	 * <pre>
	 * &lt;CONTENTCONFIGURATION&gt;
	 * 	&lt;CONTENTREPOSITORY&gt;
	 * 		&lt;PARAMETERS&gt;
	 * 			&lt;RMINAME name="name of the rmi repository resource" /&gt;
	 * 		&lt;/PARAMETERS&gt;
	 * </pre>
	 */
	public RmiRepositoryProvider() {
		try{	
			String paramRmiName = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.RMINAME";
			ConfigSingleton config = ConfigSingleton.getInstance();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"RmiRepositoryProvider::init:Spago configuration retrived: " + config);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"RmiRepositoryProvider::init:try to get the "+paramRmiName+
								" parameter from the cms configuration ");
			SourceBean paramRmiNameSB = (SourceBean)config.getAttribute(paramRmiName);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"RmiRepositoryProvider::init:cms sourcebean parameter rmi " +
								"name retrived: " + paramRmiNameSB);
			String nameRmi = (String) paramRmiNameSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"RmiRepositoryProvider::init:use "+nameRmi+" rmi name");
			ClientRepositoryFactory factory = new ClientRepositoryFactory();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					"RmiRepositoryProvider::init:ClientRepositoryFactory instantiated " + factory);
			Repository repository = factory.getRepository(nameRmi);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					"RmiRepositoryProvider::init:Repository obtained: " + repository);
	        _repo = repository;
	        
		} catch (Exception e) { 
			TracerSingleton.log(NAME_MODULE, 
		            			TracerSingleton.CRITICAL, 
		            			"RmiRepositoryProvider::init: cannot get the repository", e); 
		} finally {	}
	}
	
	/**
	 * Gets the JSR 170 repository object instance
	 * @param Repository the JSR 170 repository object instance 
	 */
	public Repository getRepository() {
		return _repo;
	}


}




